@@ -1,22 +1,9 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+import copy
 import logging
 import os
 import shutil
 import warnings
 from datetime import datetime
-from distutils import util
 from pathlib import Path
 from typing import Dict
 from typing import List
@@ -39,22 +26,18 @@ from substra.sdk.config import BackendType
 
 logger = logging.getLogger(__name__)
 
-_BACKEND_ID = "local-backend"
 _MAX_LEN_KEY_METADATA = 50
 _MAX_LEN_VALUE_METADATA = 100
-DEBUG_OWNER = "debug_owner"
 
 
 class Local(base.BaseBackend):
+
+    org_counter = 1
+
     def __init__(self, backend, *args, **kwargs):
         self._local_worker_dir = Path.cwd() / "local-worker"
         self._local_worker_dir.mkdir(exist_ok=True)
 
-        self._support_chainkeys = bool(util.strtobool(os.getenv("CHAINKEYS_ENABLED", "False")))
-        self._chainkey_dir = self._local_worker_dir / "chainkeys"
-        if self._support_chainkeys:
-            logger.info(f"Chainkeys support is on, the directory is {self._chainkey_dir}")
-
         self._debug_spawner = BackendType(os.getenv("DEBUG_SPAWNER", BackendType.LOCAL_DOCKER))
         if self._debug_spawner == BackendType.LOCAL_SUBPROCESS:
             logger.info(
@@ -67,9 +50,18 @@ class Local(base.BaseBackend):
         self._worker = compute.Worker(
             self._db,
             local_worker_dir=self._local_worker_dir,
-            support_chainkeys=self._support_chainkeys,
             debug_spawner=self._debug_spawner,
-            chainkey_dir=self._chainkey_dir,
+        )
+
+        self._org_id = f"MyOrg{Local.org_counter}MSP"
+        self._org_name = f"MyOrg{Local.org_counter}MSP"
+        Local.org_counter += 1
+        self._db.add(
+            models.Organization(
+                id=self._org_id,
+                is_current=False,  # all orgs share the same db, catch and change this value at the "get"
+                creation_date=self.__now(),
+            )
         )
 
     def __now(self):
@@ -152,7 +144,14 @@ class Local(base.BaseBackend):
         Returns:
             List[models._Model]: List of results
         """
-        return self._db.list(type_=asset_type, filters=filters, order_by=order_by, ascending=ascending)
+        results = self._db.list(type_=asset_type, filters=filters, order_by=order_by, ascending=ascending)
+        if asset_type == schemas.Type.Organization:
+            for i, item in enumerate(results):
+                if item.id == self._org_id:
+                    my_org = copy.copy(item)  # do not change the value in the db
+                    my_org.is_current = True
+                    results[i] = my_org
+        return results
 
     @staticmethod
     def _check_metadata(metadata: Optional[Dict[str, str]]):
@@ -162,20 +161,13 @@ class Local(base.BaseBackend):
             if any([len(value) > _MAX_LEN_VALUE_METADATA or len(value) == 0 for value in metadata.values()]):
                 raise exceptions.InvalidRequest("Values in metadata cannot be empty or more than 100 characters", 400)
 
-        # In debug mode, the user can define the owner of the data
-        if metadata is not None and DEBUG_OWNER in metadata:
-            owner = metadata[DEBUG_OWNER]
-        else:
-            owner = _BACKEND_ID
-        return owner
-
-    @staticmethod
-    def __compute_permissions(permissions, owner=_BACKEND_ID):
+    def __compute_permissions(self, permissions):
         """Compute the permissions
 
         If the permissions are private, the active organization is
         in the authorized ids.
         """
+        owner = self._org_id
         if permissions.public:
             permissions.authorized_ids = list()
         elif not permissions.public and owner not in permissions.authorized_ids:
@@ -205,14 +197,7 @@ class Local(base.BaseBackend):
 
     def __create_compute_plan_from_tuple(self, spec, key, in_tuples):
         # compute plan and rank
-        if not spec.compute_plan_key and spec.rank == 0:
-            #  Create a compute plan
-            compute_plan = self.__add_compute_plan(task_count=1)
-            rank = 0
-            compute_plan_key = compute_plan.key
-        elif not spec.compute_plan_key and spec.rank is not None:
-            raise substra.sdk.exceptions.InvalidRequest("invalid inputs, a new ComputePlan should have a rank 0", 400)
-        elif spec.compute_plan_key:
+        if spec.compute_plan_key is not None:
             compute_plan = self._db.get(schemas.Type.ComputePlan, spec.compute_plan_key)
             compute_plan_key = compute_plan.key
             if spec.rank is not None:
@@ -231,22 +216,17 @@ class Local(base.BaseBackend):
             compute_plan.todo_count += 1
             compute_plan.status = models.Status.waiting
 
-        else:
-            compute_plan_key = ""
+        elif not spec.compute_plan_key and (spec.rank == 0 or spec.rank is None):
+            #  Create a compute plan
+            compute_plan = self.__add_compute_plan(task_count=1)
             rank = 0
+            compute_plan_key = compute_plan.key
+        else:
+            raise substra.sdk.exceptions.InvalidRequest("invalid inputs, a new ComputePlan should have a rank 0", 400)
 
         return compute_plan_key, rank
 
     def __execute_compute_plan(self, spec, compute_plan, visited, tuples, spec_options):
-        if spec.predicttuples:
-            for predicttuple in spec.predicttuples:
-                tuple_spec = schemas.PredicttupleSpec.from_compute_plan(
-                    compute_plan_key=spec.key,
-                    spec=predicttuple,
-                )
-                visited[tuple_spec.key] = visited.get(predicttuple.traintuple_id, -1) + 1
-                tuples[tuple_spec.key] = tuple_spec
-
         if spec.testtuples:
             for testtuple in spec.testtuples:
                 tuple_spec = schemas.TesttupleSpec.from_compute_plan(
@@ -275,10 +255,6 @@ class Local(base.BaseBackend):
 
                 progress_bar.update()
 
-        compute_plan.end_date = datetime.now()
-        compute_plan.estimated_end_date = compute_plan.end_date
-        compute_plan.duration = int((compute_plan.end_date - compute_plan.start_date).total_seconds())
-
         return compute_plan
 
     def __check_data_samples(self, data_manager_key: str, data_sample_keys: List[str], allow_test_only: bool):
@@ -313,16 +289,17 @@ class Local(base.BaseBackend):
             if data_sample.test_only:
                 raise substra.exceptions.InvalidRequest("Cannot create train task with test data", 400)
 
-    def __add_algo(self, key, spec, owner, spec_options=None):
+    def __add_algo(self, key, spec, spec_options=None):
 
-        permissions = self.__compute_permissions(spec.permissions, owner)
+        permissions = self.__compute_permissions(spec.permissions)
         algo_file_path = self._db.save_file(spec.file, key)
         algo_description_path = self._db.save_file(spec.description, key)
+
         algo = models.Algo(
             key=key,
             creation_date=self.__now(),
             name=spec.name,
-            owner=owner,
+            owner=self._org_id,
             category=spec.category,
             permissions={
                 "process": {
@@ -333,26 +310,28 @@ class Local(base.BaseBackend):
             algorithm={"checksum": fs.hash_file(algo_file_path), "storage_address": algo_file_path},
             description={"checksum": fs.hash_file(algo_description_path), "storage_address": algo_description_path},
             metadata=spec.metadata if spec.metadata else dict(),
+            inputs=spec.inputs or list(),
+            outputs=spec.outputs or list(),
         )
         return self._db.add(algo)
 
     def _add_algo(self, key, spec, spec_options=None):
-        owner = self._check_metadata(spec.metadata)
-        return self.__add_algo(key, spec, owner, spec_options=spec_options)
+        self._check_metadata(spec.metadata)
+        return self.__add_algo(key, spec, spec_options=spec_options)
 
     def _add_dataset(self, key, spec, spec_options=None):
 
-        owner = self._check_metadata(spec.metadata)
+        self._check_metadata(spec.metadata)
 
-        permissions = self.__compute_permissions(spec.permissions, owner)
-        logs_permission = self.__compute_permissions(spec.logs_permission, owner)
+        permissions = self.__compute_permissions(spec.permissions)
+        logs_permission = self.__compute_permissions(spec.logs_permission)
 
         dataset_file_path = self._db.save_file(spec.data_opener, key)
         dataset_description_path = self._db.save_file(spec.description, key)
         asset = models.Dataset(
             key=key,
             creation_date=self.__now(),
-            owner=owner,
+            owner=self._org_id,
             name=spec.name,
             permissions={
                 "process": {
@@ -381,7 +360,7 @@ class Local(base.BaseBackend):
         data_sample = models.DataSample(
             key=key,
             creation_date=self.__now(),
-            owner=_BACKEND_ID,
+            owner=self._org_id,
             path=spec.path,
             data_manager_keys=spec.data_manager_keys,
             test_only=spec.test_only,
@@ -423,10 +402,6 @@ class Local(base.BaseBackend):
         # Get all the tuples and their dependencies
         tuple_graph, tuples = compute_plan_module.get_dependency_graph(spec)
 
-        # If chainkey is supported make sure it exists, else set support to False
-        if self._support_chainkeys and not (self._chainkey_dir / "organization_name_id.json").is_file():
-            logger.warning(f"No chainkeys found in {self._chainkey_dir}.")
-
         # Define the rank of each traintuple, aggregate tuple and composite tuple
         visited = graph.compute_ranks(node_graph=tuple_graph)
 
@@ -445,7 +420,7 @@ class Local(base.BaseBackend):
             canceled_count=0,
             failed_count=0,
             done_count=0,
-            owner="local",
+            owner=self._org_id,
             delete_intermediary_models=spec.clean_models if spec.clean_models is not None else False,
         )
         compute_plan = self._db.add(compute_plan)
@@ -457,7 +432,7 @@ class Local(base.BaseBackend):
     # TODO: '_add_traintuple' is too complex, consider refactoring
     def _add_traintuple(self, key, spec, spec_options=None):  # noqa: C901
         # validation
-        owner = self._check_metadata(spec.metadata)
+        self._check_metadata(spec.metadata)
 
         algo = self._db.get(schemas.Type.Algo, spec.algo_key)
         assert algo.category == schemas.AlgoCategory.simple
@@ -483,30 +458,6 @@ class Local(base.BaseBackend):
             data_manager_key=spec.data_manager_key, data_sample_keys=spec.train_data_sample_keys, allow_test_only=False
         )
 
-        # permissions
-        with_permissions = [algo, dataset] + in_tuples
-
-        public = True
-        authorized_ids = None
-        for element in with_permissions:
-            permissions = None
-            if hasattr(element, "permissions"):
-                permissions = element.permissions
-            elif hasattr(element, "train"):
-                permissions = element.train.model_permissions
-            else:
-                permissions = element.aggregate.model_permissions
-            public = public and permissions.process.public
-            if not permissions.process.public:
-                if authorized_ids is None:
-                    authorized_ids = set(permissions.process.authorized_ids)
-                else:
-                    authorized_ids = set.intersection(authorized_ids, set(permissions.process.authorized_ids))
-        if public or authorized_ids is None:
-            authorized_ids = list()
-        else:
-            authorized_ids = list(authorized_ids)
-
         compute_plan_key, rank = self.__create_compute_plan_from_tuple(spec=spec, key=key, in_tuples=in_tuples)
 
         # create model
@@ -514,19 +465,18 @@ class Local(base.BaseBackend):
             train={
                 "data_manager_key": spec.data_manager_key,
                 "data_sample_keys": spec.train_data_sample_keys,
-                "model_permissions": {
-                    "process": {"public": public, "authorized_ids": authorized_ids},
-                },
                 "models": None,
             },
             key=key,
             creation_date=self.__now(),
             category=schemas.TaskCategory.train,
             algo=algo,
-            owner=owner,
+            owner=self._org_id,
             worker=dataset.owner,
             compute_plan_key=compute_plan_key,
             rank=rank,
+            inputs=spec.inputs or [],
+            outputs=_output_from_spec(spec.outputs),
             tag=spec.tag or "",
             status=models.Status.waiting,
             metadata=spec.metadata if spec.metadata else dict(),
@@ -534,13 +484,13 @@ class Local(base.BaseBackend):
         )
 
         traintuple = self._db.add(traintuple)
-        self._worker.schedule_traintuple(traintuple)
+        self._worker.schedule_task(traintuple)
         return traintuple
 
     def _add_predicttuple(self, key, spec, spec_options=None):
 
         # validation
-        owner = self._check_metadata(spec.metadata)
+        self._check_metadata(spec.metadata)
         algo = self._db.get(schemas.Type.Algo, spec.algo_key)
 
         traintuple = self._get_parent_task(spec.traintuple_key)
@@ -570,29 +520,30 @@ class Local(base.BaseBackend):
             predict=models._Predict(
                 data_manager_key=spec.data_manager_key,
                 data_sample_keys=spec.test_data_sample_keys,
-                prediction_permissions={"process": {"public": False, "authorized_ids": [owner]}},  # TODO to verify
             ),
             key=key,
             creation_date=self.__now(),
             category=schemas.TaskCategory.predict,
             algo=algo,
-            owner=owner,
+            owner=self._org_id,
             worker=worker,
             compute_plan_key=traintuple.compute_plan_key,
-            rank=traintuple.rank,
+            rank=traintuple.rank + 1,
+            inputs=spec.inputs or [],
+            outputs=_output_from_spec(spec.outputs),
             tag=spec.tag or "",
             status=models.Status.waiting,
             metadata=spec.metadata if spec.metadata else dict(),
             parent_task_keys=[traintuple.key],
         )
         predicttuple = self._db.add(predicttuple)
-        self._worker.schedule_predicttuple(predicttuple)
+        self._worker.schedule_task(predicttuple)
         return predicttuple
 
     def _add_testtuple(self, key, spec, spec_options=None):
 
         # validation
-        owner = self._check_metadata(spec.metadata)
+        self._check_metadata(spec.metadata)
         algo = self._db.get(schemas.Type.Algo, spec.algo_key)
 
         predicttuple = self._get_parent_task(spec.predicttuple_key)
@@ -630,22 +581,24 @@ class Local(base.BaseBackend):
             creation_date=self.__now(),
             category=schemas.TaskCategory.test,
             algo=algo,
-            owner=owner,
+            owner=self._org_id,
             worker=worker,
             compute_plan_key=predicttuple.compute_plan_key,
-            rank=predicttuple.rank,
+            rank=predicttuple.rank + 1,
+            inputs=spec.inputs or [],
+            outputs=_output_from_spec(spec.outputs),
             tag=spec.tag or "",
             status=models.Status.waiting,
             metadata=spec.metadata if spec.metadata else dict(),
             parent_task_keys=[predicttuple.key],
         )
         testtuple = self._db.add(testtuple)
-        self._worker.schedule_testtuple(testtuple)
+        self._worker.schedule_task(testtuple)
         return testtuple
 
     def _add_composite_traintuple(self, key: str, spec: schemas.CompositeTraintupleSpec, spec_options=None):
         # validation
-        owner = self._check_metadata(spec.metadata)
+        self._check_metadata(spec.metadata)
         algo = self._db.get(schemas.Type.Algo, spec.algo_key)
         assert algo.category == schemas.AlgoCategory.composite
         dataset = self._db.get(schemas.Type.Dataset, spec.data_manager_key)
@@ -669,16 +622,6 @@ class Local(base.BaseBackend):
         # Compute plan
         compute_plan_key, rank = self.__create_compute_plan_from_tuple(spec, key, in_tuples)
 
-        # permissions
-        process_trunk_model_permissions = schemas.Permissions(
-            public=False, authorized_ids=spec.out_trunk_model_permissions.authorized_ids
-        )
-        trunk_model_permissions = {"process": self.__compute_permissions(process_trunk_model_permissions)}
-        if spec.in_head_model_key:
-            head_model_permissions = in_head_tuple.composite.head_permissions
-        else:
-            head_model_permissions = {"process": {"public": False, "authorized_ids": [owner]}}
-
         parent_task_keys: List[str] = list()
         if spec.in_head_model_key is not None:
             parent_task_keys.append(spec.in_head_model_key)
@@ -689,18 +632,18 @@ class Local(base.BaseBackend):
             composite=models._Composite(
                 data_manager_key=spec.data_manager_key,
                 data_sample_keys=spec.train_data_sample_keys,
-                head_permissions=head_model_permissions,
-                trunk_permissions=trunk_model_permissions,
                 models=list(),
             ),
             key=key,
             creation_date=self.__now(),
             category=schemas.TaskCategory.composite,
             algo=algo,
-            owner=owner,
+            owner=self._org_id,
             worker=dataset.owner,
             compute_plan_key=compute_plan_key,
             rank=rank,
+            inputs=spec.inputs or [],
+            outputs=_output_from_spec(spec.outputs),
             tag=spec.tag or "",
             status=models.Status.waiting,
             metadata=spec.metadata if spec.metadata else dict(),
@@ -708,7 +651,7 @@ class Local(base.BaseBackend):
         )
         composite_traintuple = self._db.add(composite_traintuple)
         if composite_traintuple.status == models.Status.waiting:
-            self._worker.schedule_traintuple(composite_traintuple)
+            self._worker.schedule_task(composite_traintuple)
         return composite_traintuple
 
     def _add_aggregatetuple(
@@ -718,23 +661,13 @@ class Local(base.BaseBackend):
         spec_options: dict = None,
     ):
         # validation
-        owner = self._check_metadata(spec.metadata)
+        self._check_metadata(spec.metadata)
         algo = self._db.get(schemas.Type.Algo, spec.algo_key)
         assert algo.category == schemas.AlgoCategory.aggregate
 
         in_tuples = list()
-        in_permissions = list()
         for in_tuple_key in spec.in_models_keys:
             in_tuple = self._get_parent_task(in_tuple_key)
-
-            if in_tuple.type_ == schemas.Type.Traintuple:
-                permissions = in_tuple.train.model_permissions
-            elif in_tuple.type_ == schemas.Type.Aggregatetuple:
-                permissions = in_tuple.aggregate.model_permissions
-            elif in_tuple.type_ == schemas.Type.CompositeTraintuple:
-                permissions = in_tuple.composite.trunk_permissions
-
-            in_permissions.append(permissions)
             in_tuples.append(in_tuple)
 
         if len(in_tuples) == 0:
@@ -743,27 +676,20 @@ class Local(base.BaseBackend):
         # Compute plan
         compute_plan_key, rank = self.__create_compute_plan_from_tuple(spec, key, in_tuples)
 
-        # Permissions
-        public = False
-        authorized_ids = set()
-        for in_permission in in_permissions:
-            if in_permission.process.public:
-                public = True
-            authorized_ids.update(in_permission.process.authorized_ids)
-
         aggregatetuple = models.Aggregatetuple(
             aggregate=models._Aggregate(
-                model_permissions={"process": {"authorized_ids": list(authorized_ids), "public": public}},
                 models=None,
             ),
             creation_date=self.__now(),
             key=key,
             category=schemas.TaskCategory.aggregate,
             algo=algo,
-            owner=owner,
-            worker=owner,
+            owner=self._org_id,
+            worker=spec.worker,
             compute_plan_key=compute_plan_key,
             rank=rank,
+            inputs=spec.inputs or [],
+            outputs=_output_from_spec(spec.outputs),
             tag=spec.tag or "",
             status=models.Status.waiting,
             metadata=spec.metadata if spec.metadata else dict(),
@@ -771,10 +697,47 @@ class Local(base.BaseBackend):
         )
         aggregatetuple = self._db.add(aggregatetuple)
         if aggregatetuple.status == models.Status.waiting:
-            self._worker.schedule_traintuple(aggregatetuple)
+            self._worker.schedule_task(aggregatetuple)
         return aggregatetuple
 
+    def _check_inputs_outputs(self, spec, algo_key):
+        algo = self._db.get(schemas.Type.Algo, algo_key)
+        spec_inputs = spec.inputs or list()
+        spec_outputs = spec.outputs or dict()
+
+        error_msg = ""
+        for algo_input in algo.inputs:
+            spec_values = [x for x in spec_inputs if x.identifier == algo_input.identifier]
+            if len(spec_values) == 0 and not algo_input.optional:
+                error_msg += f"  - The input {algo_input.identifier} is not optional according to the algo.\n"
+            if len(spec_values) > 1 and not algo_input.multiple:
+                error_msg += f"  - The input {algo_input.identifier} is not multiple according to the algo.\n"
+
+        if not set([x.identifier for x in spec_inputs]).issubset(set([x.identifier for x in algo.inputs])):
+            error_msg += "  - There are keys in the spec inputs that have not been declared in the algo inputs.\n"
+
+        spec_output_keys = set([x for x in spec_outputs])
+        algo_output_keys = set([x.identifier for x in algo.outputs])
+        if not spec_output_keys == algo_output_keys:
+            error_msg += (
+                "  - The identifiers in the spec outputs and the algo outputs must match:"
+                f"{spec_output_keys.symmetric_difference(algo_output_keys)}.\n"
+            )
+
+        for output_performance in (x for x in algo.outputs if x.kind == schemas.AssetKind.performance):
+            if not spec_outputs[output_performance.identifier].permissions.public:
+                error_msg += '  - invalid task output "performance": a PERFORMANCE output should be public.'
+                break
+
+        if len(error_msg) > 0:
+            raise exceptions.InvalidRequest(f"There are errors in the inputs / outputs fields:\n{error_msg}", "OE0101")
+
     def add(self, spec, spec_options=None, key=None):
+
+        algo_key = getattr(spec, "algo_key", None)
+        if algo_key is not None:
+            self._check_inputs_outputs(spec=spec, algo_key=algo_key)
+
         # find dynamically the method to call to create the asset
         method_name = f"_add_{spec.__class__.type_.value}"
         if spec.is_many():
@@ -808,17 +771,19 @@ class Local(base.BaseBackend):
             data_samples.append(data_sample)
         return data_sample_keys
 
-    def download(self, asset_type, url_field_path, key, destination):
+    def download(self, asset_type, url_field_path, key, destination_file):
         if self._db.is_local(key, asset_type):
             asset = self._db.get(type_=asset_type, key=key)
             # Get the field containing the path to the file.
             file_path = asset
             for field in url_field_path.split("."):
                 file_path = getattr(file_path, field, None)
-            # Copy the file to the destination folder
-            shutil.copyfile(file_path, destination)
+            # Copy the file to the destination file
+            shutil.copyfile(file_path, destination_file)
         else:
-            self._db.remote_download(asset_type, url_field_path, key, destination)
+            self._db.remote_download(asset_type, url_field_path, key, destination_file)
+
+        return destination_file
 
     def download_model(self, key, destination_file):
         if self._db.is_local(key, schemas.Type.Model):
@@ -827,6 +792,8 @@ class Local(base.BaseBackend):
         else:
             self._db.remote_download_model(key, destination_file)
 
+        return destination_file
+
     def download_logs(self, tuple_key: str, destination_file: str = None) -> NoReturn:
         raise NotImplementedError("Logs of tuples ran in local mode are not accessible")
 
@@ -841,14 +808,33 @@ class Local(base.BaseBackend):
             return self._db.get_remote_description(asset_type, key)
 
     def organization_info(self):
-        return {"type": "local backend"}
+        return models.OrganizationInfo(
+            host="http://fakeurl.fake",
+            organization_id=self._org_id,
+            organization_name=self._org_name,
+            config=models.OrganizationInfoConfig(
+                model_export_enabled=True,
+            ),
+            channel="channel1",
+            version=substra.__version__,
+            orchestrator_version="fake",
+        )
 
     def cancel_compute_plan(self, key):
         # Execution is synchronous in the local backend so this
         # function does not make sense.
         raise NotImplementedError
 
-    def add_compute_plan_tuples(self, spec: schemas.UpdateComputePlanSpec, spec_options: dict = None):
+    def update(self, key, spec, spec_options=None):
+        asset_type = spec.__class__.type_
+        asset = self.get(asset_type, key)
+        data = asset.dict()
+        data.update(spec.dict())
+        updated_asset = models.SCHEMA_TO_MODEL[asset_type](**data)
+        self._db.update(updated_asset)
+        return
+
+    def add_compute_plan_tuples(self, spec: schemas.UpdateComputePlanTuplesSpec, spec_options: dict = None):
         key = spec.key
         compute_plan = self._db.get(schemas.Type.ComputePlan, key)
         # Get all the new tuples and their dependencies
@@ -868,8 +854,12 @@ class Local(base.BaseBackend):
             asset_type=schemas.Type.CompositeTraintuple,
             filters=filters,
         )
+        cp_predicttuple = self.list(
+            asset_type=schemas.Type.Predicttuple,
+            filters=filters,
+        )
         visited = dict()
-        for tuple_ in cp_traintuples + cp_aggregatetuples + cp_composite_traintuples:
+        for tuple_ in cp_traintuples + cp_aggregatetuples + cp_composite_traintuples + cp_predicttuple:
             visited[tuple_.key] = tuple_.rank
 
         # Update the tuple graph with the tuples already in the CP
@@ -878,3 +868,14 @@ class Local(base.BaseBackend):
 
         compute_plan = self.__execute_compute_plan(spec, compute_plan, visited, tuples, spec_options)
         return compute_plan
+
+
+def _output_from_spec(outputs: Dict[str, schemas.ComputeTaskOutputSpec]) -> Dict[str, models.ComputeTaskOutput]:
+    """Convert a list of schemas.ComputeTaskOuput to a list of models.ComputeTaskOutput"""
+    return {
+        identifier: models.ComputeTaskOutput(
+            permissions=models.Permissions(process=output.permissions), transient=output.is_transient, value=None
+        )
+        # default isNone (= outputs are not computed yet)
+        for identifier, output in outputs.items()
+    }
