@@ -1,16 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 import logging
 import pathlib
 import shutil
@@ -23,7 +10,6 @@ from substra.sdk import schemas
 from substra.sdk.backends.local import db
 from substra.sdk.backends.remote import backend
 
-_LOCAL_KEY = "local_"
 logger = logging.getLogger(__name__)
 
 
@@ -34,7 +20,7 @@ class DataAccess:
     """
 
     def __init__(self, remote_backend: typing.Optional[backend.Remote], local_worker_dir: pathlib.Path):
-        self._db = db.InMemoryDb()
+        self._db = db.get_db()
         self._remote = remote_backend
         self._tmp_dir = tempfile.TemporaryDirectory(prefix=str(local_worker_dir) + "/")
 
@@ -200,7 +186,8 @@ class DataAccess:
         return tmp_file
 
     def update(self, asset):
-        return self._db.update(asset)
+        self._db.update(asset)
+        return
 
     # TODO: '_get_local_model' is too complex, consider refactoring
     def _get_local_model(self, key):  # noqa: C901
