@@ -1,17 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import json
 import logging
 import math
@@ -220,14 +206,7 @@ class Remote(base.BaseBackend):
             batches = [tuples]
 
         for batch in batches:
-            try:
-                self._add_tuples(batch, spec_options)
-            except exceptions.AlreadyExists:
-                logger.warning(
-                    "Skipping already submitted tasks, probably because of a timeout error. "
-                    "Check that the compute plan has the right number of tasks once the submission is complete."
-                )
-                continue
+            self._add_tuples(batch, spec_options)
 
     def _add_tuples(self, batch, spec_options):
         batch_data = []
@@ -241,6 +220,34 @@ class Remote(base.BaseBackend):
             json={"tasks": batch_data},
         )
 
+    def _update(self, asset, key, data, files=None):
+        data = deepcopy(data)  # make a deep copy for avoiding modification by reference
+        if files:
+            kwargs = {
+                "data": {
+                    "json": json.dumps(data),
+                },
+                "files": files,
+            }
+
+        else:
+            kwargs = {
+                "json": data,
+            }
+
+        return self._client.update(
+            asset.to_server(),
+            key,
+            retry_timeout=self._retry_timeout,
+            **kwargs,
+        )
+
+    def update(self, key, spec, spec_options=None):
+        spec_options = spec_options or {}
+        asset_type = spec.__class__.type_
+        with spec.build_request_kwargs(**spec_options) as (data, files):
+            return self._update(asset_type, key, data, files=files)
+
     def add_compute_plan_tuples(self, spec, spec_options):
         # Remove auto_batching and batch_size from spec_options
         auto_batching = spec_options.pop(AUTO_BATCHING, False)
@@ -301,7 +308,7 @@ class Remote(base.BaseBackend):
 
     def organization_info(self):
         response = self._client.get_data(f"{self._client.base_url}/info/")
-        return response.json()
+        return models.OrganizationInfo(**response.json())
 
     def cancel_compute_plan(self, key: str) -> None:
         self._client.request(
