@@ -1,17 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 from substra.sdk import exceptions
 from substra.sdk import graph
 from substra.sdk import schemas
@@ -57,7 +43,6 @@ def get_dependency_graph(spec: schemas._BaseComputePlanSpec):
 
     if spec.composite_traintuples:
         for compositetuple in spec.composite_traintuples:
-            assert not compositetuple.out_trunk_model_permissions.public
             _insert_into_graph(
                 tuple_graph=tuple_graph,
                 tuple_id=compositetuple.composite_traintuple_id,
@@ -75,7 +60,18 @@ def get_dependency_graph(spec: schemas._BaseComputePlanSpec):
                 rank=None,
                 spec=compositetuple,
             )
-
+    if spec.predicttuples:
+        for predicttuple in spec.predicttuples:
+            _insert_into_graph(
+                tuple_graph=tuple_graph,
+                tuple_id=predicttuple.predicttuple_id,
+                in_model_ids=[predicttuple.traintuple_id],
+            )
+            tuples[predicttuple.predicttuple_id] = schemas.PredicttupleSpec.from_compute_plan(
+                compute_plan_key=spec.key,
+                rank=None,
+                spec=predicttuple,
+            )
     return tuple_graph, tuples
 
 
@@ -96,16 +92,6 @@ def get_tuples(spec):
     # are not their actual ranks in the compute plan)
     id_ranks = graph.compute_ranks(node_graph=tuple_graph, node_to_ignore=already_created_ids)
 
-    # Add the predicttuples to 'visited' to take them into account in the batches
-    if spec.predicttuples:
-        for predicttuple in spec.predicttuples:
-            tuple_spec = schemas.PredicttupleSpec.from_compute_plan(
-                compute_plan_key=spec.key,
-                spec=predicttuple,
-            )
-            id_ranks[tuple_spec.key] = id_ranks.get(predicttuple.traintuple_id, -1) + 1
-            tuples[tuple_spec.key] = tuple_spec
-
     # Add the testtuples to 'visited' to take them into account in the batches
     if spec.testtuples:
         for testtuple in spec.testtuples:
