@@ -1,16 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 import abc
 import enum
 import re
@@ -21,10 +8,12 @@ from typing import List
 from typing import Optional
 from typing import Union
 
+import pydantic
 from pydantic import AnyUrl
 from pydantic import DirectoryPath
 from pydantic import FilePath
 from pydantic import root_validator
+from pydantic.fields import Field
 
 from substra.sdk import schemas
 
@@ -175,6 +164,19 @@ class Dataset(_Model):
         return ["key", "name", "owner", "permissions", "compute_plan_key", "algo_key", "data_sample_key"]
 
 
+class AlgoInput(_Model):
+    identifier: str
+    kind: schemas.AssetKind
+    optional: bool
+    multiple: bool
+
+
+class AlgoOutput(_Model):
+    identifier: str
+    kind: schemas.AssetKind
+    multiple: bool
+
+
 class Algo(_Model):
     key: str
     name: str
@@ -183,6 +185,8 @@ class Algo(_Model):
     metadata: Dict[str, str]
     category: schemas.AlgoCategory
     creation_date: datetime
+    inputs: List[AlgoInput]
+    outputs: List[AlgoOutput]
 
     description: _File
     algorithm: _File
@@ -193,6 +197,33 @@ class Algo(_Model):
     def allowed_filters() -> List[str]:
         return ["key", "name", "owner", "permissions", "compute_plan_key", "dataset_key", "data_sample_key"]
 
+    @pydantic.validator("inputs", pre=True)
+    def dict_input_to_list(cls, v):  # noqa: N805
+        if isinstance(v, dict):
+            # Transform the inputs dict to a list
+            return [
+                AlgoInput(
+                    identifier=identifier,
+                    kind=algo_input["kind"],
+                    optional=algo_input["optional"],
+                    multiple=algo_input["multiple"],
+                )
+                for identifier, algo_input in v.items()
+            ]
+        else:
+            return v
+
+    @pydantic.validator("outputs", pre=True)
+    def dict_output_to_list(cls, v):  # noqa: N805
+        if isinstance(v, dict):
+            # Transform the outputs dict to a list
+            return [
+                AlgoOutput(identifier=identifier, kind=algo_output["kind"], multiple=algo_output["multiple"])
+                for identifier, algo_output in v.items()
+            ]
+        else:
+            return v
+
 
 class InModel(schemas._PydanticConfig):
     """In model of a traintuple, aggregate or composite traintuple"""
@@ -206,7 +237,7 @@ class OutModel(schemas._PydanticConfig):
     model of a composite traintuple"""
 
     key: str
-    category: ModelType
+    category: ModelType  # TODO: delete this field
     compute_task_key: str
     address: Optional[InModel]
     permissions: Permissions
@@ -215,6 +246,31 @@ class OutModel(schemas._PydanticConfig):
 
     type_: ClassVar[str] = schemas.Type.Model
 
+    @staticmethod
+    def allowed_filters() -> List[str]:
+        return ["key", "compute_task_key", "owner", "permissions"]
+
+
+class InputRef(schemas._PydanticConfig):
+    identifier: str
+    asset_key: Optional[str]
+    parent_task_key: Optional[str]
+    parent_task_output_identifier: Optional[str]
+
+    # either (asset_key) or (parent_task_key, parent_task_output_identifier) must be specified
+    _check_asset_key_or_parent_ref = pydantic.root_validator(allow_reuse=True)(schemas.check_asset_key_or_parent_ref)
+
+
+class ComputeTaskOutput(schemas._PydanticConfig):
+    """Specification of a compute task input"""
+
+    permissions: Permissions
+    value: Optional[Union[float, OutModel, List[OutModel]]]  # performance or Model or multiple model
+    is_transient: bool = Field(False, alias="transient")
+
+    class Config:
+        allow_population_by_field_name = True
+
 
 class _GenericTraintuple(_Model):
     key: str
@@ -228,6 +284,8 @@ class _GenericTraintuple(_Model):
     rank: Optional[int]
     parent_task_keys: List[str]
     parent_tasks: Optional[List[Union["Traintuple", "CompositeTraintuple", "Aggregatetuple", "Predicttuple"]]] = list()
+    inputs: List[InputRef]
+    outputs: Dict[str, ComputeTaskOutput]
     tag: str
     creation_date: datetime
     start_date: Optional[datetime]
@@ -260,8 +318,6 @@ class _Composite(schemas._PydanticConfig):
     data_manager_key: str
     data_manager: Optional[Dataset] = None
     data_sample_keys: List[str]
-    head_permissions: Permissions
-    trunk_permissions: Permissions
     models: Optional[List[OutModel]]
 
     _check_data_manager_key = root_validator(allow_reuse=True)(check_data_manager_key)
@@ -271,14 +327,12 @@ class _Train(schemas._PydanticConfig):
     data_manager_key: str
     data_manager: Optional[Dataset] = None
     data_sample_keys: List[str]
-    model_permissions: Permissions
     models: Optional[List[OutModel]]
 
     _check_data_manager_key = root_validator(allow_reuse=True)(check_data_manager_key)
 
 
 class _Aggregate(schemas._PydanticConfig):
-    model_permissions: Permissions
     models: Optional[List[OutModel]]
 
 
@@ -286,7 +340,6 @@ class _Predict(schemas._PydanticConfig):
     data_manager_key: str
     data_manager: Optional[Dataset] = None
     data_sample_keys: List[str]
-    prediction_permissions: Permissions
     models: Optional[List[OutModel]]
 
     _check_data_manager_key = root_validator(allow_reuse=True)(check_data_manager_key)
@@ -405,6 +458,22 @@ class Organization(schemas._PydanticConfig):
     is_current: bool
     creation_date: datetime
 
+    type_: ClassVar[str] = schemas.Type.Organization
+
+
+class OrganizationInfoConfig(schemas._PydanticConfig):
+    model_export_enabled: bool
+
+
+class OrganizationInfo(schemas._PydanticConfig):
+    host: AnyUrl
+    organization_id: str
+    organization_name: str
+    config: OrganizationInfoConfig
+    channel: str
+    version: str
+    orchestrator_version: str
+
 
 SCHEMA_TO_MODEL = {
     schemas.Type.Aggregatetuple: Aggregatetuple,
