@@ -1,18 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-import abc
 import contextlib
 import enum
 import json
@@ -24,64 +9,34 @@ from typing import List
 from typing import Optional
 
 import pydantic
+from pydantic.fields import Field
 
 from substra.sdk import utils
 
-# TODO create a sub-package schemas:
-# types
-# inputs
-# outputs
-
-
 _SERVER_NAMES = {
     "dataset": "data_manager",
 }
 
 
-ALGO_INPUTS_PER_CATEGORY = {
-    "ALGO_SIMPLE": {
-        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
-        "model": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
-        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
-    },
-    "ALGO_AGGREGATE": {
-        "model": {"kind": "ASSET_MODEL", "multiple": True, "optional": False},
-    },
-    "ALGO_COMPOSITE": {
-        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
-        "local": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
-        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
-        "shared": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
-    },
-    "ALGO_METRIC": {
-        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
-        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
-        "predictions": {"kind": "ASSET_MODEL", "multiple": False, "optional": False},
-    },
-    "ALGO_PREDICT": {
-        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
-        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
-        "model": {"kind": "ASSET_MODEL", "multiple": False, "optional": False},
-        "shared": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
-    },
-}
+class StaticInputIdentifier(str, enum.Enum):
+    chainkeys = "chainkeys"
+    datasamples = "datasamples"
+    opener = "opener"
 
-ALGO_OUTPUTS_PER_CATEGORY = {
-    "ALGO_SIMPLE": {
-        "model": {"kind": "ASSET_MODEL", "multiple": False},
-    },
-    "ALGO_AGGREGATE": {
-        "model": {"kind": "ASSET_MODEL", "multiple": False},
-    },
-    "ALGO_COMPOSITE": {
-        "local": {"kind": "ASSET_MODEL", "multiple": False},
-        "shared": {"kind": "ASSET_MODEL", "multiple": False},
-    },
-    "ALGO_METRIC": {
-        "performance": {"kind": "ASSET_PERFORMANCE", "multiple": False},
-    },
-    "ALGO_PREDICT": {"predictions": {"kind": "ASSET_MODEL", "multiple": False}},
-}
+    @classmethod
+    def values(cls):
+        return set(item.value for item in cls)
+
+    @classmethod
+    def has_value(cls, value):
+        return value in cls.values()
+
+
+class AssetKind(str, enum.Enum):
+    model = "ASSET_MODEL"
+    performance = "ASSET_PERFORMANCE"
+    data_manager = "ASSET_DATA_MANAGER"
+    data_sample = "ASSET_DATA_SAMPLE"
 
 
 class Type(enum.Enum):
@@ -136,7 +91,7 @@ class _PydanticConfig(pydantic.BaseModel):
         extra = "ignore"
 
 
-class _Spec(_PydanticConfig, abc.ABC):
+class _Spec(_PydanticConfig):
     """Asset creation specification base class."""
 
     # pretty print
@@ -222,12 +177,48 @@ class DataSampleSpec(_Spec):
             yield data, None
 
 
+def check_asset_key_or_parent_ref(cls, values):
+    """Check that either (asset key) or (parent_task_key, parent_task_output_identifier) are set, but not both."""
+
+    has_asset_key = bool(values.get("asset_key"))
+    has_parent = bool(values.get("parent_task_key")) and bool(values.get("parent_task_output_identifier"))
+
+    if has_asset_key != has_parent:  # xor
+        return values
+
+    raise ValueError("either asset_key or both (parent_task_key, parent_task_output_identifier) must be provided")
+
+
+class InputRef(_PydanticConfig):
+    """Specification of a compute task input"""
+
+    identifier: str
+    asset_key: Optional[str]
+    parent_task_key: Optional[str]
+    parent_task_output_identifier: Optional[str]
+
+    # either (asset_key) or (parent_task_key, parent_task_output_identifier) must be specified
+    _check_asset_key_or_parent_ref = pydantic.root_validator(allow_reuse=True)(check_asset_key_or_parent_ref)
+
+
+class ComputeTaskOutputSpec(_PydanticConfig):
+    """Specification of a compute task output"""
+
+    permissions: Permissions
+    is_transient: Optional[bool] = Field(False, alias="transient")
+
+    class Config:
+        allow_population_by_field_name = True
+
+
 class _ComputePlanComputeTaskSpec(_Spec):
     """Specification of a compute task inside a compute plan specification"""
 
     algo_key: str
     tag: Optional[str]
     metadata: Optional[Dict[str, str]]
+    inputs: Optional[List[InputRef]]
+    outputs: Optional[Dict[str, ComputeTaskOutputSpec]]
 
 
 class ComputePlanTraintupleSpec(_ComputePlanComputeTaskSpec):
@@ -258,7 +249,6 @@ class ComputePlanCompositeTraintupleSpec(_ComputePlanComputeTaskSpec):
     train_data_sample_keys: List[str]
     in_head_model_id: Optional[str]
     in_trunk_model_id: Optional[str]
-    out_trunk_model_permissions: Permissions
 
 
 class ComputePlanPredicttupleSpec(_ComputePlanComputeTaskSpec):
@@ -280,7 +270,7 @@ class ComputePlanTesttupleSpec(_ComputePlanComputeTaskSpec):
     test_data_sample_keys: List[str]
 
 
-class _BaseComputePlanSpec(_Spec, abc.ABC):
+class _BaseComputePlanSpec(_Spec):
     key: str
     traintuples: Optional[List[ComputePlanTraintupleSpec]]
     composite_traintuples: Optional[List[ComputePlanCompositeTraintupleSpec]]
@@ -309,12 +299,20 @@ class ComputePlanSpec(_BaseComputePlanSpec):
         yield data, None
 
 
-class UpdateComputePlanSpec(_BaseComputePlanSpec):
-    """Specification for updating a compute plan"""
+class UpdateComputePlanTuplesSpec(_BaseComputePlanSpec):
+    """Specification for updating a compute plan's tuples"""
 
     pass
 
 
+class UpdateComputePlanSpec(_Spec):
+    """Specification for updating a compute plan"""
+
+    name: str
+
+    type_: typing.ClassVar[Type] = Type.ComputePlan
+
+
 class DatasetSpec(_Spec):
     """Specification for creating a dataset"""
 
@@ -335,6 +333,64 @@ class DatasetSpec(_Spec):
         )
 
 
+class UpdateDatasetSpec(_Spec):
+    """Specification for updating a dataset"""
+
+    name: str
+
+    type_: typing.ClassVar[Type] = Type.Dataset
+
+
+class AlgoInputSpec(_Spec):
+    identifier: str
+    multiple: bool
+    optional: bool
+    kind: AssetKind
+
+    @pydantic.root_validator
+    def _check_identifiers(cls, values):  # noqa: N805
+        """Checks that the multiplicity and the optionality of a data manager is always set to False"""
+        if values.get("kind") == AssetKind.data_manager:
+            if values.get("multiple"):
+                raise ValueError("Data manager input can't be multiple.")
+            if values.get("optional"):
+                raise ValueError("Data manager input can't be optional.")
+            if values.get("identifier") != StaticInputIdentifier.opener:
+                raise ValueError(
+                    f"Data manager input identifier must be `{StaticInputIdentifier.opener}` "
+                    f"but was set to {values['identifier']}"
+                )
+        elif values.get("kind") == AssetKind.data_sample:
+            if values.get("identifier") != StaticInputIdentifier.datasamples:
+                raise ValueError(
+                    f"Data sample input identifier must be `{StaticInputIdentifier.datasamples}` "
+                    f"but was set to {values['identifier']}"
+                )
+
+        else:
+            if StaticInputIdentifier.has_value(values.get("identifier")):
+                raise ValueError(
+                    f"Inputs of kind {values.get('kind')} cannot have an identifier among the flowing: "
+                    f"{StaticInputIdentifier.values()}"
+                )
+
+        return values
+
+
+class AlgoOutputSpec(_Spec):
+    identifier: str
+    kind: AssetKind
+    multiple: bool
+
+    @pydantic.root_validator
+    def _check_performance(cls, values):  # noqa: N805
+        """Checks that the performance is always set to False"""
+        if values.get("kind") == AssetKind.performance and values.get("multiple"):
+            raise ValueError("Performance can't be multiple.")
+
+        return values
+
+
 class AlgoSpec(_Spec):
     """Specification for creating an algo"""
 
@@ -344,6 +400,8 @@ class AlgoSpec(_Spec):
     permissions: Permissions
     metadata: Optional[Dict[str, str]]
     category: AlgoCategory
+    inputs: Optional[List[AlgoInputSpec]] = None
+    outputs: Optional[List[AlgoOutputSpec]] = None
 
     type_: typing.ClassVar[Type] = Type.Algo
 
@@ -353,12 +411,14 @@ class AlgoSpec(_Spec):
         # Serialize and deserialize to prevent errors eg with pathlib.Path
         data = json.loads(self.json(exclude_unset=True))
 
-        # Waiting for algo inputs/outputs to be added by the public API,
-        # these data are added on the fly to the request, without being exposed to the user.
         # Computed fields using `@property` are not dumped when `exclude_unset` flag is enabled,
         # this is why we need to reimplement this custom function.
-        data["inputs"] = ALGO_INPUTS_PER_CATEGORY[self.category]
-        data["outputs"] = ALGO_OUTPUTS_PER_CATEGORY[self.category]
+        data["inputs"] = (
+            {input.identifier: input.dict(exclude={"identifier"}) for input in self.inputs} if self.inputs else {}
+        )
+        data["outputs"] = (
+            {output.identifier: output.dict(exclude={"identifier"}) for output in self.outputs} if self.outputs else {}
+        )
 
         if self.Meta.file_attributes:
             with utils.extract_files(data, self.Meta.file_attributes) as (data, files):
@@ -373,12 +433,22 @@ class AlgoSpec(_Spec):
         )
 
 
+class UpdateAlgoSpec(_Spec):
+    """Specification for updating an algo"""
+
+    name: str
+
+    type_: typing.ClassVar[Type] = Type.Algo
+
+
 class _TupleSpec(_Spec):
     key: str = pydantic.Field(default_factory=lambda: str(uuid.uuid4()))
     tag: Optional[str]
     compute_plan_key: Optional[str]
     metadata: Optional[Dict[str, str]]
     algo_key: str
+    inputs: Optional[List[InputRef]]
+    outputs: Optional[Dict[str, ComputeTaskOutputSpec]]
 
     @contextlib.contextmanager
     def build_request_kwargs(self):
@@ -387,6 +457,8 @@ class _TupleSpec(_Spec):
         data = json.loads(self.json(exclude_unset=True))
         data["key"] = self.key
         data["category"] = self.category
+        data["inputs"] = [input.dict() for input in self.inputs] if self.inputs else []
+        data["outputs"] = {k: v.dict(by_alias=True) for k, v in self.outputs.items()} if self.outputs else {}
         yield data, None
 
 
@@ -410,6 +482,8 @@ class TraintupleSpec(_TupleSpec):
             data_manager_key=spec.data_manager_key,
             train_data_sample_keys=spec.train_data_sample_keys,
             in_models_keys=spec.in_models_ids or list(),
+            inputs=spec.inputs,
+            outputs=spec.outputs,
             tag=spec.tag,
             compute_plan_key=compute_plan_key,
             rank=rank,
@@ -437,6 +511,8 @@ class AggregatetupleSpec(_TupleSpec):
             algo_key=spec.algo_key,
             worker=spec.worker,
             in_models_keys=spec.in_models_ids or list(),
+            inputs=spec.inputs,
+            outputs=spec.outputs,
             tag=spec.tag,
             compute_plan_key=compute_plan_key,
             rank=rank,
@@ -451,7 +527,6 @@ class CompositeTraintupleSpec(_TupleSpec):
     train_data_sample_keys: List[str]
     in_head_model_key: Optional[str]
     in_trunk_model_key: Optional[str]
-    out_trunk_model_permissions: Permissions
     rank: Optional[int]
     category: TaskCategory = pydantic.Field(TaskCategory.composite, const=True)
 
@@ -469,10 +544,8 @@ class CompositeTraintupleSpec(_TupleSpec):
             train_data_sample_keys=spec.train_data_sample_keys,
             in_head_model_key=spec.in_head_model_id,
             in_trunk_model_key=spec.in_trunk_model_id,
-            out_trunk_model_permissions={
-                "public": spec.out_trunk_model_permissions.public,
-                "authorized_ids": spec.out_trunk_model_permissions.authorized_ids,
-            },
+            inputs=spec.inputs,
+            outputs=spec.outputs,
             tag=spec.tag,
             compute_plan_key=compute_plan_key,
             rank=rank,
@@ -492,16 +565,21 @@ class PredicttupleSpec(_TupleSpec):
     type_: typing.ClassVar[Type] = Type.Predicttuple
 
     @classmethod
-    def from_compute_plan(cls, compute_plan_key: str, spec: ComputePlanPredicttupleSpec) -> "PredicttupleSpec":
+    def from_compute_plan(
+        cls, compute_plan_key: str, rank: int, spec: ComputePlanPredicttupleSpec
+    ) -> "PredicttupleSpec":
         return PredicttupleSpec(
             key=spec.predicttuple_id,
             algo_key=spec.algo_key,
             traintuple_key=spec.traintuple_id,
+            inputs=spec.inputs,
+            outputs=spec.outputs,
             tag=spec.tag,
             data_manager_key=spec.data_manager_key,
             test_data_sample_keys=spec.test_data_sample_keys,
             compute_plan_key=compute_plan_key,
             metadata=spec.metadata,
+            rank=rank,
         )
 
 
@@ -520,6 +598,8 @@ class TesttupleSpec(_TupleSpec):
         return TesttupleSpec(
             algo_key=spec.algo_key,
             predicttuple_key=spec.predicttuple_id,
+            inputs=spec.inputs,
+            outputs=spec.outputs,
             tag=spec.tag,
             data_manager_key=spec.data_manager_key,
             test_data_sample_keys=spec.test_data_sample_keys,
