@@ -1,16 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 METRIC = {
     "key": "d5002e1c-d50b-d5de-5341-df8a7b7d11b6",
     "name": "Skin Lesion Classification Challenge",
@@ -30,6 +17,12 @@ METRIC = {
         "checksum": "52eea19fccfde2b12e30f4d16fd7d48f035e03210ad5804616f71799c4bdc0de",
         "storage_address": "http://testserver/metric/7b75b728-9b9b-45fe-a867-4bb1525b7b5d/file/",
     },
+    "inputs": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+        "predictions": {"kind": "ASSET_MODEL", "optional": False, "multiple": False},
+    },
+    "outputs": {"performance": {"kind": "ASSET_PERFORMANCE", "multiple": False}},
 }
 
 DATASET = {
@@ -83,8 +76,72 @@ ALGO = {
         "checksum": "aa8d43bf6e3341b0034a2e396451ab731ccca95a4c1d4f65a4fcd30f9081ec7d",
         "storage_address": "http://testserver/algo/17f98afc-2b82-4ce9-b232-1a471633d020/file/",
     },
+    "inputs": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+        "model": {"kind": "ASSET_MODEL", "optional": True, "multiple": True},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+    },
+    "outputs": {"model": {"kind": "ASSET_MODEL", "multiple": False}},
 }
 
+
+ALGO_PREDICT = {
+    "key": "681eedb9-db00-4480-a66f-63c86cc20280",
+    "name": "def637b111f2495bb6b4771644d2409e_test_traintuple_data_samples_relative_order - Algo Predict",
+    "owner": "MyOrg1MSP",
+    "creation_date": "2021-08-24T13:36:07.393646367Z",
+    "permissions": {
+        "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+        "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+    },
+    "metadata": {},
+    "category": "ALGO_PREDICT",
+    "description": {
+        "checksum": "756589d5971c421a388a751d533ab8ce09715c93040e9e8fff1365e831545aa2",
+        "storage_address": "http://testserver/algo/681eedb9-db00-4480-a66f-63c86cc20280/description/",
+    },
+    "algorithm": {
+        "checksum": "aa8d43bf6e3341b0034a2e396451ab731ccca95a4c1d4f65a4fcd30f9081ec7d",
+        "storage_address": "http://testserver/algo/681eedb9-db00-4480-a66f-63c86cc20280/file/",
+    },
+    "inputs": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+        "model": {"kind": "ASSET_MODEL", "optional": False, "multiple": False},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+        "shared": {"kind": "ASSET_MODEL", "optional": True, "multiple": False},
+    },
+    "outputs": {"predictions": {"kind": "ASSET_MODEL", "multiple": False}},
+}
+
+
+ALGO_METRIC = {
+    "key": "6a8ada2e-740f-46f4-af0f-11376763ed72",
+    "name": "def637b111f2495bb6b4771644d2409e_test_traintuple_data_samples_relative_order - Algo Metric",
+    "owner": "MyOrg1MSP",
+    "creation_date": "2021-08-24T13:36:07.393646367Z",
+    "permissions": {
+        "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+        "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+    },
+    "metadata": {},
+    "category": "ALGO_METRIC",
+    "description": {
+        "checksum": "756589d5971c421a388a751d533ab8ce09715c93040e9e8fff1365e831545aa2",
+        "storage_address": "http://testserver/algo/6a8ada2e-740f-46f4-af0f-11376763ed72/description/",
+    },
+    "algorithm": {
+        "checksum": "aa8d43bf6e3341b0034a2e396451ab731ccca95a4c1d4f65a4fcd30f9081ec7d",
+        "storage_address": "http://testserver/algo/6a8ada2e-740f-46f4-af0f-11376763ed72/file/",
+    },
+    "inputs": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+        "predictions": {"kind": "ASSET_MODEL", "optional": False, "multiple": False},
+    },
+    "outputs": {"performance": {"kind": "ASSET_PERFORMANCE", "multiple": False}},
+}
+
+
 TRAINTUPLE = {
     "key": "30c283be-d385-424e-94a6-4d8538275260",
     "category": "TASK_TRAIN",
@@ -107,6 +164,12 @@ TRAINTUPLE = {
             "checksum": "aa8d43bf6e3341b0034a2e396451ab731ccca95a4c1d4f65a4fcd30f9081ec7d",
             "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/17f98afc-2b82-4ce9-b232-1a471633d020/file/",  # noqa: E501
         },
+        "inputs": {
+            "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+            "model": {"kind": "ASSET_MODEL", "optional": True, "multiple": True},
+            "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+        },
+        "outputs": {"model": {"kind": "ASSET_MODEL", "multiple": False}},
     },
     "owner": "MyOrg1MSP",
     "creation_date": "2021-08-24T13:36:07.393646367Z",
@@ -119,6 +182,22 @@ TRAINTUPLE = {
     "tag": "",
     "start_date": "2021-10-12T09:28:06.947765800Z",
     "end_date": "2021-10-12T09:30:04.705947400Z",
+    "inputs": [
+        {"identifier": "opener", "asset_key": "a67b9387-fd80-429a-bc2f-034fac430b0f"},
+        {"identifier": "datasamples", "asset_key": "3180e12c-a821-434a-ad8a-a341076c7f98"},
+        {"identifier": "datasamples", "asset_key": "21bb59ca-abd4-4154-b04a-44a92556a078"},
+        {"identifier": "datasamples", "asset_key": "67512646-2464-4521-84de-419b1b307d30"},
+        {"identifier": "datasamples", "asset_key": "8c5e430b-cb6b-4f4e-95da-499b523d9f5b"},
+    ],
+    "outputs": {
+        "model": {
+            "permissions": {
+                "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+                "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+            },
+            "transient": False,
+        },
+    },
     "train": {
         "data_manager_key": "a67b9387-fd80-429a-bc2f-034fac430b0f",
         "data_sample_keys": [
@@ -127,10 +206,6 @@ TRAINTUPLE = {
             "67512646-2464-4521-84de-419b1b307d30",
             "8c5e430b-cb6b-4f4e-95da-499b523d9f5b",
         ],
-        "model_permissions": {
-            "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
-            "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
-        },
         "models": [
             {
                 "key": "6f0ee20a328044fb89e70ee5d219fa0c",
@@ -173,6 +248,8 @@ AGGREGATETUPLE = {
             "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
         },
         "metadata": {"foo": "bar"},
+        "inputs": {"model": {"kind": "ASSET_MODEL", "optional": False, "multiple": True}},
+        "outputs": {"model": {"kind": "ASSET_MODEL", "multiple": False}},
     },
     "owner": "MyOrg1MSP",
     "creation_date": "2021-08-24T13:36:07.393646367Z",
@@ -185,11 +262,17 @@ AGGREGATETUPLE = {
     "tag": "My super tag",
     "start_date": "2021-10-12T09:28:06.947765800Z",
     "end_date": "2021-10-12T09:30:04.705947400Z",
-    "aggregate": {
-        "model_permissions": {
-            "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
-            "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+    "inputs": [],
+    "outputs": {
+        "model": {
+            "permissions": {
+                "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+                "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+            },
+            "transient": False,
         },
+    },
+    "aggregate": {
         "models": [
             {
                 "key": "5f0ee20a328044fb89e70ee5d219fa0b",
@@ -232,6 +315,16 @@ COMPOSITE_TRAINTUPLE = {
             "checksum": "51bd5fe2e7f087b203d1b4a73f3b3276b9fde96a0fff9c1f5984de96e4675d59",
             "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/c663b6e6-dd62-49fb-afe8-191fa7627a64/file/",  # noqa: E501
         },
+        "inputs": {
+            "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+            "local": {"kind": "ASSET_MODEL", "optional": True, "multiple": False},
+            "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+            "shared": {"kind": "ASSET_MODEL", "optional": True, "multiple": False},
+        },
+        "outputs": {
+            "local": {"kind": "ASSET_MODEL", "multiple": False},
+            "shared": {"kind": "ASSET_MODEL", "multiple": False},
+        },
     },
     "owner": "MyOrg1MSP",
     "creation_date": "2021-08-24T13:36:07.393646367Z",
@@ -244,17 +337,29 @@ COMPOSITE_TRAINTUPLE = {
     "tag": "",
     "start_date": "2021-10-12T09:28:06.947765800Z",
     "end_date": "2021-10-12T09:30:04.705947400Z",
+    "inputs": [
+        {"identifier": "opener", "asset_key": "1ae64423-aa99-4b8a-8660-61a56e4ca42d"},
+        {"identifier": "datasamples", "asset_key": "1288d38c-dec2-4433-bc40-ec17f99f522a"},
+    ],
+    "outputs": {
+        "shared": {
+            "permissions": {
+                "process": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
+                "download": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
+            },
+            "transient": False,
+        },
+        "local": {
+            "permissions": {
+                "process": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
+                "download": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
+            },
+            "transient": False,
+        },
+    },
     "composite": {
         "data_manager_key": "1ae64423-aa99-4b8a-8660-61a56e4ca42d",
         "data_sample_keys": ["1288d38c-dec2-4433-bc40-ec17f99f522a"],
-        "head_permissions": {
-            "process": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
-            "download": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
-        },
-        "trunk_permissions": {
-            "process": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
-            "download": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
-        },
         "models": [
             {
                 "key": "ac4a1a82ced4419b83f4bba78c024b94",
@@ -312,6 +417,16 @@ COMPOSITE_TRAINTUPLE_DOING = {
             "checksum": "51bd5fe2e7f087b203d1b4a73f3b3276b9fde96a0fff9c1f5984de96e4675d59",
             "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/c663b6e6-dd62-49fb-afe8-191fa7627a64/file/",  # noqa: E501
         },
+        "inputs": {
+            "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+            "local": {"kind": "ASSET_MODEL", "optional": True, "multiple": False},
+            "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+            "shared": {"kind": "ASSET_MODEL", "optional": True, "multiple": False},
+        },
+        "outputs": {
+            "local": {"kind": "ASSET_MODEL", "multiple": False},
+            "shared": {"kind": "ASSET_MODEL", "multiple": False},
+        },
     },
     "owner": "MyOrg1MSP",
     "creation_date": "2021-08-24T13:36:07.393646367Z",
@@ -322,27 +437,39 @@ COMPOSITE_TRAINTUPLE_DOING = {
     "rank": 0,
     "parent_task_keys": [],
     "tag": "",
+    "inputs": [
+        {"identifier": "opener", "asset_key": "1ae64423-aa99-4b8a-8660-61a56e4ca42d"},
+        {"identifier": "datasamples", "asset_key": "1288d38c-dec2-4433-bc40-ec17f99f522a"},
+    ],
+    "outputs": {
+        "shared": {
+            "permissions": {
+                "process": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
+                "download": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
+            },
+            "transient": True,
+        },
+        "local": {
+            "permissions": {
+                "process": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
+                "download": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
+            },
+            "transient": False,
+        },
+    },
     "composite": {
         "data_manager_key": "1ae64423-aa99-4b8a-8660-61a56e4ca42d",
         "data_sample_keys": ["1288d38c-dec2-4433-bc40-ec17f99f522a"],
-        "head_permissions": {
-            "process": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
-            "download": {"public": False, "authorized_ids": ["MyOrg2MSP"]},
-        },
-        "trunk_permissions": {
-            "process": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
-            "download": {"public": False, "authorized_ids": ["MyOrg1MSP", "MyOrg2MSP"]},
-        },
         "models": None,
     },
 }
 
-TESTTUPLE = {
+PREDICTTUPLE = {
     "key": "afa9c7b1-21b5-4e58-a098-79de4ecede35",
-    "category": "TASK_TEST",
+    "category": "TASK_PREDICT",
     "algo": {
-        "key": "17f98afc-2b82-4ce9-b232-1a471633d020",
-        "name": "def637b111f2495bb6b4771644d2409e_test_traintuple_data_samples_relative_order - Algo 0",
+        "key": "681eedb9-db00-4480-a66f-63c86cc20280",
+        "name": "def637b111f2495bb6b4771644d2409e_test_traintuple_data_samples_relative_order - Algo Predict681eedb9-db00-4480-a66f-63c86cc20280",  # noqa: E501
         "owner": "MyOrg1MSP",
         "creation_date": "2021-08-24T13:36:07.428974249Z",
         "permissions": {
@@ -350,15 +477,22 @@ TESTTUPLE = {
             "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
         },
         "metadata": {},
-        "category": "ALGO_SIMPLE",
+        "category": "ALGO_PREDICT",
         "description": {
             "checksum": "756589d5971c421a388a751d533ab8ce09715c93040e9e8fff1365e831545aa2",
-            "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/17f98afc-2b82-4ce9-b232-1a471633d020/description/",  # noqa: E501
+            "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/681eedb9-db00-4480-a66f-63c86cc20280/description/",  # noqa: E501
         },
         "algorithm": {
             "checksum": "aa8d43bf6e3341b0034a2e396451ab731ccca95a4c1d4f65a4fcd30f9081ec7d",
-            "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/17f98afc-2b82-4ce9-b232-1a471633d020/file/",  # noqa: E501
+            "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/681eedb9-db00-4480-a66f-63c86cc20280/file/",  # noqa: E501
         },
+        "inputs": {
+            "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+            "model": {"kind": "ASSET_MODEL", "optional": False, "multiple": False},
+            "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+            "shared": {"kind": "ASSET_MODEL", "optional": True, "multiple": False},
+        },
+        "outputs": {"predictions": {"kind": "ASSET_MODEL", "multiple": False}},
     },
     "owner": "MyOrg1MSP",
     "creation_date": "2021-08-24T13:36:07.393646367Z",
@@ -371,6 +505,97 @@ TESTTUPLE = {
     "tag": "",
     "start_date": "2021-10-12T09:28:06.947765800Z",
     "end_date": "2021-10-12T09:30:04.705947400Z",
+    "inputs": [
+        {"identifier": "opener", "asset_key": "a67b9387-fd80-429a-bc2f-034fac430b0f"},
+        {"identifier": "datasamples", "asset_key": "3180e12c-a821-434a-ad8a-a341076c7f98"},
+        {"identifier": "datasamples", "asset_key": "21bb59ca-abd4-4154-b04a-44a92556a078"},
+        {
+            "identifier": "model",
+            "parent_task_key": "30c283be-d385-424e-94a6-4d8538275260",
+            "parent_task_output_identifier": "model",
+        },
+    ],
+    "outputs": {
+        "predictions": {
+            "permissions": {
+                "process": {"public": True, "authorized_ids": []},
+                "download": {"public": True, "authorized_ids": []},
+            },
+            "transient": False,
+        },
+    },
+    "predict": {
+        "data_manager_key": "a67b9387-fd80-429a-bc2f-034fac430b0f",
+        "data_sample_keys": [
+            "3180e12c-a821-434a-ad8a-a341076c7f98",
+            "21bb59ca-abd4-4154-b04a-44a92556a078",
+        ],
+        "prediction_permissions": {
+            "process": {"public": True, "authorized_ids": []},
+            "download": {"public": True, "authorized_ids": []},
+        },
+    },
+}
+
+TESTTUPLE = {
+    "key": "afa9c7b1-21b5-4e58-a098-79de4ecede35",
+    "category": "TASK_TEST",
+    "algo": {
+        "key": "6a8ada2e-740f-46f4-af0f-11376763ed72",
+        "name": "def637b111f2495bb6b4771644d2409e_test_traintuple_data_samples_relative_order - Algo Metric",
+        "owner": "MyOrg1MSP",
+        "creation_date": "2021-08-24T13:36:07.428974249Z",
+        "permissions": {
+            "process": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+            "download": {"public": True, "authorized_ids": ["MyOrg1MSP"]},
+        },
+        "metadata": {},
+        "category": "ALGO_METRIC",
+        "description": {
+            "checksum": "756589d5971c421a388a751d533ab8ce09715c93040e9e8fff1365e831545aa2",
+            "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/6a8ada2e-740f-46f4-af0f-11376763ed72/description/",  # noqa: E501
+        },
+        "algorithm": {
+            "checksum": "aa8d43bf6e3341b0034a2e396451ab731ccca95a4c1d4f65a4fcd30f9081ec7d",
+            "storage_address": "http://backend-org-1-substra-backend-server.org-1:8000/algo/6a8ada2e-740f-46f4-af0f-11376763ed72/file/",  # noqa: E501
+        },
+        "inputs": {
+            "datasamples": {"kind": "ASSET_DATA_SAMPLE", "optional": False, "multiple": True},
+            "opener": {"kind": "ASSET_DATA_MANAGER", "optional": False, "multiple": False},
+            "predictions": {"kind": "ASSET_MODEL", "optional": False, "multiple": False},
+        },
+        "outputs": {"performance": {"kind": "ASSET_PERFORMANCE", "multiple": False}},
+    },
+    "owner": "MyOrg1MSP",
+    "creation_date": "2021-08-24T13:36:07.393646367Z",
+    "compute_plan_key": "a428d7a3-7458-421d-a818-7a2448225b8d",
+    "metadata": {},
+    "status": "STATUS_DONE",
+    "worker": "MyOrg1MSP",
+    "rank": 0,
+    "parent_task_keys": ["05cf2e58-5862-444b-82de-cdaaead97c53"],
+    "tag": "",
+    "start_date": "2021-10-12T09:28:06.947765800Z",
+    "end_date": "2021-10-12T09:30:04.705947400Z",
+    "inputs": [
+        {"identifier": "opener", "asset_key": "a67b9387-fd80-429a-bc2f-034fac430b0f"},
+        {"identifier": "datasamples", "asset_key": "3180e12c-a821-434a-ad8a-a341076c7f98"},
+        {"identifier": "datasamples", "asset_key": "21bb59ca-abd4-4154-b04a-44a92556a078"},
+        {
+            "identifier": "predictions",
+            "parent_task_key": "05cf2e58-5862-444b-82de-cdaaead97c53",
+            "parent_task_output_identifier": "predictions",
+        },
+    ],
+    "outputs": {
+        "performance": {
+            "permissions": {
+                "process": {"public": True, "authorized_ids": []},
+                "download": {"public": True, "authorized_ids": []},
+            },
+            "transient": False,
+        },
+    },
     "test": {
         "data_manager_key": "a67b9387-fd80-429a-bc2f-034fac430b0f",
         "data_sample_keys": [
