@@ -1,17 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import os
 from unittest.mock import patch
 
@@ -27,14 +13,14 @@ from ..utils import mock_response
 
 
 @pytest.mark.parametrize(
-    "asset_type, filename",
+    "asset_type",
     [
-        ("dataset", "opener.py"),
-        ("algo", "algo.tar.gz"),
-        ("model", "model_foo"),
+        ("dataset"),
+        ("algo"),
+        ("model"),
     ],
 )
-def test_download_asset(asset_type, filename, tmp_path, client, mocker):
+def test_download_asset(asset_type, tmp_path, client, mocker):
     item = getattr(datastore, asset_type.upper())
     responses = [
         mock_response(item),  # metadata
@@ -43,9 +29,8 @@ def test_download_asset(asset_type, filename, tmp_path, client, mocker):
     m = mock_requests_responses(mocker, "get", responses)
 
     method = getattr(client, f"download_{asset_type}")
-    method("foo", tmp_path)
+    temp_file = method("foo", tmp_path)
 
-    temp_file = str(tmp_path) + "/" + filename
     assert os.path.exists(temp_file)
     m.assert_called()
 
