@@ -1,17 +1,3 @@
-# Copyright 2018 Owkin, inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import pytest
 
 from substra.sdk import exceptions
@@ -28,12 +14,14 @@ from ..utils import mock_requests
     [
         "dataset",
         "algo",
+        "predicttuple",
         "testtuple",
         "traintuple",
         "aggregatetuple",
         "composite_traintuple",
         "compute_plan",
         "data_sample",
+        "model",
     ],
 )
 def test_list_asset(asset_type, client, mocker):
@@ -54,12 +42,14 @@ def test_list_asset(asset_type, client, mocker):
     [
         ("dataset", {"permissions": ["foo", "bar"]}),
         ("algo", {"owner": ["foo", "bar"]}),
+        ("predicttuple", {"rank": [1, 3]}),
         ("testtuple", {"rank": [1, 3]}),
         ("traintuple", {"key": ["foo", "bar"]}),
         ("aggregatetuple", {"worker": ["foo", "bar"]}),
         ("composite_traintuple", {"owner": ["foo", "bar"]}),
         ("compute_plan", {"name": "foo"}),
         ("compute_plan", {"status": [models.ComputePlanStatus.done.value]}),
+        ("model", {"owner": ["MyOrg1MSP"]}),
     ],
 )
 def test_list_asset_with_filters(asset_type, filters, client, mocker):
@@ -90,6 +80,7 @@ def test_list_asset_with_filters_failure(client, mocker):
 @pytest.mark.parametrize(
     "asset_type",
     [
+        "predicttuple",
         "testtuple",
         "traintuple",
         "aggregatetuple",
